@@ -4,7 +4,7 @@
 //! The profiler's configured resolution is the same as the core clock. The cycle count clock is
 //! free-running, so overflows are likely if you have long running functions to profile.
 //! To mitigate this, one can use the `extended` feature, which extends the resolution of
-//! the counter from [`u32`] to [`u64`] using the [`SysTick`] exception. It is set
+//! the counter from 24 bit to [`u32`] or [`u64`] using the [`SysTick`] exception. It is set
 //! to expire just before overflow, so you can expect an exception to fire every 2**24
 //! clock cycles.
 //!
@@ -29,12 +29,30 @@
 //! embedded_profiling::profile("print_profile", || println!("Hello, world"));
 //! ```
 //!
+//! ## Features
+//!
+//! ### `extended`
+//!
+//! as discussed above, extend the native resolution of 24 bits to either 32 or 64 bits
+//! using the [`SysTick`] exception. The exception fires ever 2**24 clock cycles.
+//!
+//! ### `container-u64`
+//!
+//! enables the `container-u64` feature in [`embedded-profiling`](embedded_profiling). Use
+//! a [`u64`] as the time storage type instead of [`u32`] for longer running profiling.
+//!
+//! ### `proc-macros`
+//!
+//! enables the `proc-macros` feature in [`embedded-profiling`](embedded_profiling). Enables
+//! the [`embedded_profiling::profile_function`] procedural macro.
+//!
 //! [`SYST`]: cortex_m::peripheral::SYST
 //! [`SysTick`]: `cortex_m::peripheral::scb::Exception::SysTick`
-
+//! [`embedded_profiling::profile_function`]: https://docs.rs/embedded-profiling/latest/embedded_profiling/attr.profile_function.html
 #![cfg_attr(not(test), no_std)]
+
 use cortex_m::peripheral::{syst::SystClkSource, SYST};
-use embedded_profiling::{EPContainer, EPInstant, EPSnapshot, EmbeddedProfiler};
+use embedded_profiling::{EPContainer, EPInstant, EPInstantGeneric, EPSnapshot, EmbeddedProfiler};
 
 #[cfg(feature = "extended")]
 /// Tracker of `systick` cycle count overflows to extend systick's 24 bit timer
@@ -74,80 +92,26 @@ impl<const FREQ: u32> SysTickProfiler<FREQ> {
 
         Self { systick }
     }
-
-    /// binary GCD function stolen from wikipedia, made const
-    const fn gcd(mut u: EPContainer, mut v: EPContainer) -> EPContainer {
-        // Base cases: gcd(n, 0) = gcd(0, n) = n
-        if u == 0 {
-            return v;
-        } else if v == 0 {
-            return u;
-        }
-
-        // Using identities 2 and 3:
-        // gcd(2ⁱ u, 2ʲ v) = 2ᵏ gcd(u, v) with u, v odd and k = min(i, j)
-        // 2ᵏ is the greatest power of two that divides both u and v
-        let i = u.trailing_zeros();
-        u >>= i;
-        let j = v.trailing_zeros();
-        v >>= j;
-
-        // min(i, j);
-        let k = if i <= j { i } else { j };
-
-        loop {
-            // u and v are odd at the start of the loop
-            // debug_assert!(u % 2 == 1, "u = {} is even", u);
-            // debug_assert!(v % 2 == 1, "v = {} is even", v);
-
-            // Swap if necessary so u <= v
-            if u > v {
-                // swap(&mut u, &mut v);
-                let tmp = u;
-                u = v;
-                v = tmp;
-            }
-
-            // Using identity 4 (gcd(u, v) = gcd(|v-u|, min(u, v))
-            v -= u;
-
-            // Identity 1: gcd(u, 0) = u
-            // The shift by k is necessary to add back the 2ᵏ factor that was removed before the loop
-            if v == 0 {
-                return u << k;
-            }
-
-            // Identity 3: gcd(u, 2ʲ v) = gcd(u, v) (u is known to be odd)
-            v >>= v.trailing_zeros();
-        }
-    }
-
-    /// Reduce the fraction we need to convert between 1µs precision and whatever our core clock is running at
-    pub(crate) const fn reduced_fraction() -> (EPContainer, EPContainer) {
-        let gcd = Self::gcd(1_000_000, FREQ as EPContainer) as EPContainer;
-        (1_000_000 / gcd, FREQ as EPContainer / gcd)
-    }
 }
 
 impl<const FREQ: u32> EmbeddedProfiler for SysTickProfiler<FREQ> {
     fn read_clock(&self) -> EPInstant {
         let raw_ticks = SYST::get_current();
         #[allow(unused_mut)]
-        let mut count = (SYSTICK_RELOAD - raw_ticks) as EPContainer;
+        let mut count = EPContainer::from(SYSTICK_RELOAD - raw_ticks);
 
         #[cfg(feature = "extended")]
         {
             /// the resolution of [`systick`](cortex_m::peripheral::SYST), 2**24
-            const SYSTICK_RESOLUTION: EPContainer = 16777216;
+            const SYSTICK_RESOLUTION: EPContainer = 16_777_216;
 
             // add on the number of times we've rolled over (systick resolution is 2**24)
             let rollover_count =
-                ROLLOVER_COUNT.load(core::sync::atomic::Ordering::Acquire) as EPContainer;
+                EPContainer::from(ROLLOVER_COUNT.load(core::sync::atomic::Ordering::Acquire));
             count += rollover_count * SYSTICK_RESOLUTION;
         }
 
-        let (red_num, red_denom) = Self::reduced_fraction();
-        EPInstant::from_ticks(count * red_num / red_denom)
+        embedded_profiling::convert_instant(EPInstantGeneric::<1, FREQ>::from_ticks(count))
     }
 
     fn log_snapshot(&self, snapshot: &EPSnapshot) {
